%{
/* fmcfg:
 *		Greg Lee, July 2001
 */
#include "fm.h"

#define PROMPT 256
static int state = -1;

static int
	l_bold = -1,
	l_italic = -1,
	l_math = -1,
	l_ttype = -1,
	l_slanty = -1,
	l_super = -1,
	l_sub = -1,
	l_ipa = -1,
	l_prompt = -1,
	l_bitalic = -1,
	l_sans = -1,
	l_musix = -1,
	l_roman = -1;

static unsigned char cfg_attribute = ROMAN;
static void upd(int a);
static int max(int a, int b);

%}

%option noyywrap
%option nounput

%s F

wh	[ \t]*
dim	[ \t=]*("+"|"-")?[0-9]*("."[0-9]+)?[ ]*(in|pt|em|ex|pc|cm)[ \t]*
na	[^a-z]

%%

"bold:"			state = BOLD;
"italic:"		state = ITALIC;
"slanted:"		state = SLANTY;
"math:"			state = MATH;
"ttype:"		state = TTYPE;
"bitalic:"		state = BITALIC;
"musix:"		state = MUSIX;
"sans:"			state = SANS;
"super:"		state = SUPER;
"sub:"			state = SUB;
"ipa:"			state = IPA;
"roman:"		state = ROMAN;
"prompt:"		state = PROMPT;

"yellow"		upd(C_YELLOW);
"cyan"			upd(C_CYAN);
"blue"			upd(C_BLUE);
"green"			upd(C_GREEN);
"red"			upd(C_RED);
"magenta"		upd(C_MAGENTA);
"white"			upd(C_WHITE);

"bright"		upd(M_BRIGHT);
"reverse"		upd(M_REVERSE);
"blink"			upd(M_BLINK);
"dim"			upd(M_DIM);
"underscore"		upd(M_UNDERSCORE);

"unicode"[ \t]+"console"  alwayskflag = TRUE;

"flags:"		BEGIN(F);

<F>"-"[1abceFijmnNquUxyz]+  {
	int c, i;
	for (i = 1; (c=yytext[i]); i++)
		switch(c) {
			case 'a': staflag = TRUE; break;
#ifndef STANDALONETREE
			case 'i': stiflag = TRUE; break;
			case 'j': jflag = FALSE; break;
			case 'c': cflag = TRUE; break;
			case 'F': fflag = TRUE; break;
			case 'x': xflag = TRUE; break;
			case 'y': yflag = TRUE; break;
			case 'z': zflag = TRUE; break;
			case 'm': mflag = TRUE; break;
			case 'n': nflag = TRUE; break;
			case 'b': oflag = TRUE; uflag = FALSE; break;
			case 'e': uflag = oflag = aflag = kflag = FALSE; break;
#endif
			case 'u': kflag = TRUE; break;
			case 'U': kflag = FALSE; break;
			case '1': gflag = FALSE; break;
#ifndef STANDALONETREE
			case 'N': liflag = obflag = obsflag = -1;
				  jflag = FALSE; hlevel = 0; break;
			case 'q': vflag = FALSE; break;
#endif
		         default: break;
		}
}

<F>"-"[lhtdfw][ \t]*("-")?[0-9]+ {
#ifndef STANDALONETREE
	switch(yytext[1]) {
			case 'w': hsize = max(0, atoi(yytext+2)); break;
			case 'h': hlevel = max(0, atoi(yytext+2)); break;
			case 'l': offset = atoi(yytext+2); break;
			case 't': tabsize = max(1, atoi(yytext+2)); break;
			case 'd': outdev = max(0, atoi(yytext+2)); break;
			case 'f': bfont = max(0, atoi(yytext+2)); break;
		         default: break;
	}
#endif
}

\\"tenrm"/{na}  {
#ifndef STANDALONETREE
	pointsize = 10;
#endif
}
\\"hoffset"{dim}/{na}  {
#ifndef STANDALONETREE
		offset = dimen(yytext+8, TRUE, TRUE, FALSE);
		offset += 12;
#endif
}
\\"hsize"{dim}/{na}  {
#ifndef STANDALONETREE
		hsize = dimen(yytext+6, TRUE, FALSE, FALSE);
#endif
}
\\"vsize"{dim}/{na}  {
#ifndef STANDALONETREE
		vsize = dimen(yytext+6, FALSE, FALSE, FALSE);
#endif
}
\\"parindent"{dim}/{na}  {
#ifndef STANDALONETREE
		parindent = dimen(yytext+10, TRUE, FALSE, FALSE);
#endif
}
\\"baselineskip"{dim}/{na}  {
#ifndef STANDALONETREE
		baselineskip = dimen(yytext+13, FALSE, FALSE, FALSE);
		ps_baselineskip = dimen(yytext+13, FALSE, FALSE, TRUE);
#endif
}
\\"voffset"{dim}/{na}  {
#ifndef STANDALONETREE
		ps_voffset = dimen(yytext+8, FALSE, TRUE, TRUE);
#endif
}
\\"parskip"{dim}/{na}  {
#ifndef STANDALONETREE
		parskip = dimen(yytext+8, FALSE, FALSE, FALSE);
#endif
}
\\"raggedright"/{na}  {
#ifndef STANDALONETREE
	jflag = FALSE;
#endif
}
\\"nopagenumbers"/{na}  {
#ifndef STANDALONETREE
	nopagenumbers = TRUE;
#endif
}
\\p/{na}	cfg_attribute = IPA;
\\ipa/{na}	cfg_attribute = IPA;
"$"		cfg_attribute = MATH;
\\rm/{na}	cfg_attribute = ROMAN;
\\let[ \t]+[^ \t\r\n]=(\"[0-9a-fA-F]+|\'[0-7]+|[0-9]{2,}|[^ \t\r\n]) {
#ifndef STANDALONETREE
	int i, oldc, newc;
	for (i = 4; yytext[i] == ' ' || yytext[i] == '\t'; i++) ;
	oldc = yytext[i];
	i += 2;
	if (!yytext[i+1]) newc = yytext[i];
	else {
		if (yytext[i] == '"') newc = strtol(yytext+i+1, 0, 16);
		else if (yytext[i] == '\'') newc = strtol(yytext+i+1, 0, 8);
		else newc = strtol(yytext+i, 0, 10);
	}
	if (newc >= 0 && newc < 256) {
		if (is_ipa(cfg_attribute)) let[2][oldc] = newc;
		else if (cfg_attribute == MATH) let[1][oldc] = newc;
		else if (is_roman(cfg_attribute)) let[0][oldc] = newc;
	}
#endif
}
\n	BEGIN(0);

.	;

%%

static void upd(int a)
{
	int c = -1, m = -1;

	switch(state) {
		case ROMAN:  if ( (c=l_roman) == -1) l_roman = 0; break;
		case BOLD:   if ( (c=l_bold) == -1) l_bold = 0; break;
		case ITALIC: if ( (c=l_italic) == -1) l_italic = 0; break;
		case TTYPE:  if ( (c=l_ttype) == -1) l_ttype = 0; break;
		case MATH:   if ( (c=l_math) == -1) l_math = 0; break;
		case SUPER:  if ( (c=l_super) == -1) l_super = 0; break;
		case SUB:    if ( (c=l_sub) == -1) l_sub = 0; break;
		case IPA:    if ( (c=l_ipa) == -1) l_ipa = 0; break;
		case SLANTY: if ( (c=l_slanty) == -1) l_slanty = 0; break;
		case BITALIC:if ( (c=l_bitalic) == -1) l_bitalic = 0; break;
		case SANS:   if ( (c=l_sans) == -1) l_sans = 0; break;
		case MUSIX:  if ( (c=l_musix) == -1) l_musix = 0; break;
		case PROMPT: if ( (c=l_prompt) == -1) l_prompt = 0; break;
		default: break;
	}
	if (c > 0) {
		m = c >> 4;
		c &= 0x07;
		if (m > 0 && (a>>4) > 0) return;
		if (c > 0 && (a & 0x07) > 0) return;
	}

	switch(state) {
		case ROMAN:  l_roman |= a; break;
		case BOLD:   l_bold |= a; break;
		case ITALIC: l_italic |= a; break;
		case TTYPE:  l_ttype |= a; break;
		case MATH:   l_math |= a; break;
		case SUPER:  l_super |= a; break;
		case SUB:    l_sub |= a; break;
		case IPA:    l_ipa |= a; break;
		case SLANTY: l_slanty |= a; break;
		case BITALIC:l_bitalic |= a; break;
		case MUSIX:  l_musix |= a; break;
		case SANS:   l_sans |= a; break;
		case PROMPT: l_prompt |= a; break;
		default: break;
	}
}

int fmcfg()
{
	char   *str;
	char   *path;
	extern char    *getenv();

	if ((str = getenv ("HOME")) == (char *) NULL) {
		(void) fprintf (stderr, "fm: HOME env. variable not set\n");
		exit (1);
	}
	path = strcpy( (char *)malloc(strlen(str)+11), str);
	strcat(path, "/.fmconfig");

	cfgin = fopen(path, "r");

	free(path);

	if (cfgin != NULL) {
		cfglex ();
		fclose(cfgin);
		if (l_roman >= 0) c_roman = l_roman;
		if (l_bold >= 0) c_bold = l_bold;
		if (l_italic >= 0) c_italic = l_italic;
		if (l_ttype >= 0) c_ttype = l_ttype;
		if (l_math >= 0) c_math = l_math;
		if (l_super >= 0) c_super = l_super;
		if (l_sub >= 0) c_sub = l_sub;
		if (l_ipa >= 0) c_ipa = l_ipa;
		if (l_slanty >= 0) c_slanty = l_slanty;
		if (l_bitalic >= 0) c_bitalic = l_bitalic;
		if (l_musix >= 0) c_musix = l_musix;
		if (l_sans >= 0) c_sans = l_sans;
		if (l_prompt >= 0) c_prompt = l_prompt;
	}

	return 0;
}

static int max(int a, int b)
{
	return ((a > b) ? a : b);
}				/* max */
